#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <bsp.h>
#include <netinterface.h>
#include <string.h>
#include <system.h>

/*******************************************************************************
 * MOD54417 Network Operation Overview                                         *
 * ------------------------------------                                        *
 * The MOD54417's software drivers are designed to be fully transparent and    *
 * perform all necessary configurations using the same function calls as the   *
 * MOD54415. So, what are the relevant function calls and what do they do      *
 * differently between the MOD54415 and the MOD54417?                          *
 *                                                                             *
 * Hardware Initialization                                                     *
 * ------------------------                                                    *
 * The hardware initialization for the network stack ocurrs in                 *
 * InitializeStack(). At the bottom, this function breaks out of the main      *
 * system library when it calls AddEthernetInterface(). Inside this call, there*
 * is a check that determines if the software is running on a MOD54415 or a    *
 * MOD54417. If it is running on a MOD54417 it then checks to see if it should *
 * be running in switched mode or independent mode. By the time the function   *
 * returns, the network hardware is configured accordingly and the interface(s)*
 * are added to the stack (if the device is in switched mode, only one         *
 * interface is created, due to the nature of the switch). Add2ndEthernet()    *
 * should never be called in normal application development. If the module     *
 * should switch between switched and independent modes, the config record     *
 * must be updated and then the module be reset.                               *
 *                                                                             *
 * Network Configurations                                                      *
 * -----------------------                                                     *
 * When the ethernet interfaces are operating in independent mode, the second  *
 * ethernet interface will get its configuration from ConfigRecord number      *
 * CONFIG_IF_ID_ETHERNET2. It will appear under IPSetup. In most ways, the     *
 * second interface will behave as the first, with a few minor exceptions.     *
 *                                                                             *
 * 1. The link status functions will now operate on the driver objects.        *
 *    Example:  EtherLink() becomes enet0.EtherLink() or enet1.EtherLink()     *
 * 2. All the calls with Ethernet* (EthernetIP, EthernetIpMask, EthernetIpGate,*
 *    EthernetDNS) will need to need to be changed to work with the            *
 *    NetInterface functions for querying the different addresses, or obtaining*
 *    the InterfaceBlock for the correct interface. These can be obtained by   *
 *    the oppropriate call as follows:                                         *
 *       InterfaceBlock *pifb = GetInterFaceBlock( enet0.myInterfaceNumber );  *
 *    Once you have the block, the you can then set the individual addresses.  *
 *                                                                             *
 * GetDHCPAddressIfNecessary                                                   *
 * --------------------------                                                  *
 * GetDHCPAddressIfNecessary has been modified in the default case. The default*
 * argument is -1. When this value is passed, GetDHCPAddressIfNecessary will   *
 * start DHCP negotiations on all network interfaces that do not already have  *
 * either, a valid IP address, a DHCP object associated, or an AutoIP object   *
 * associated.                                                                 *
 *******************************************************************************/

#define FLAG_USE_ESWITCH (0x02)

extern "C" {
void UserMain(void * pd);
void IpQuestion( char *pMsg, unsigned long *pVal );
void ReadLine(char *buffer, int maxlen);
void ShowChange( char *pMsg, unsigned long newval, unsigned long oldval );
}


const char * AppName="MOD54417-Multi-Intf";

// Configures the Ethernet switch for switch or independent mode
void ConfigureESwitch( bool enableSwitch )
{
    ConfigRecord *rec = GetValidConfig(CONFIG_IF_ID_ETHERNET);
    ConfigRecord cr;
    if (!rec) {return;}
    memcpy( &cr, rec, sizeof( cr ) );

    if (enableSwitch) {
        cr.m_Flags |= FLAG_USE_ESWITCH;
    }
    else {
        cr.m_Flags &= ~FLAG_USE_ESWITCH;
    }
    UpdateConfigRecord_Num(&cr, CONFIG_IF_ID_ETHERNET);
    ForceReboot();
}

void PrintInterfaceInfo( InterfaceBlock *pifb, int intf_num )
{
    iprintf("Interface %d:\r\n", intf_num);
    iprintf("    Name:    %s\r\n", pifb->InterfaceName);
    iprintf("    IP:      %I\r\n", pifb->netIP);
    iprintf("    NetMask: %I\r\n", pifb->netIpMask);
    iprintf("    Gateway: %I\r\n", pifb->netIpGate);
    iprintf("    DNS:     %I\r\n", pifb->netDNS);
    iprintf("    Method:  %s\r\n",
            pifb->dhcpClient ? "DHCP" : (pifb->AutoIPC ? "AutoIP" : "Static"));
}

void ShowNetworkInterfaces()
{
    int intf_num = GetFirstInterface();
    InterfaceBlock *pifb;
    while (intf_num) {
        pifb = GetInterFaceBlock(intf_num);
        PrintInterfaceInfo( pifb, intf_num );
        iprintf("\r\n");
        intf_num = GetnextInterface( intf_num );
    }
}

void SetupDialog( int enet_num )
{
    ConfigRecord cr;
    char buffer[20];
    iprintf( "\n" );
    memcpy( &cr, &gConfigRec, sizeof( cr ) );
    IpQuestion( "IP Address", &cr.ip_Addr );
    IpQuestion( "NetMask   ", &cr.ip_Mask );
    IpQuestion( "Gateway   ", &cr.ip_GateWay );
    IpQuestion( "DNS Server", &cr.ip_DNS_server );
    iprintf( "\n" );

    buffer[0] = 0;

    while ( buffer[0] == 0 )
    {
        iprintf( "Save Changes? [Y/N]" );
        ReadLine( buffer, 80 );
    }

    if ( ( buffer[0] == 'Y' ) || ( buffer[0] == 'y' ) )
    {
        iprintf( "Changing..\n" );
        ShowChange( "IP Address ", cr.ip_Addr, gConfigRec.ip_Addr );
        ShowChange( "NetMask    ", cr.ip_Mask, gConfigRec.ip_Mask );
        ShowChange( "Gateway    ", cr.ip_GateWay, gConfigRec.ip_GateWay );
        ShowChange( "DNS Server ", cr.ip_DNS_server, gConfigRec.ip_DNS_server );
        if (enet_num == 1) {
            UpdateConfigRecord_Num( &cr, CONFIG_IF_ID_ETHERNET2 );
        }
        else {
            UpdateConfigRecord_Num( &cr, CONFIG_IF_ID_ETHERNET );
        }
    }

    iprintf( "\n" );
}

void DisplayMenu()
{
    iprintf("\r\n------ Main Menu ------\r\n");
    iprintf("D - Enable Independent Mode and Reset\r\n");
    iprintf("E - Enable Ethernet Switch and Reset\r\n");
    iprintf("I - Setup Ethernet 0\r\n");
    iprintf("J - Setup Ethernet 1\r\n");
    iprintf("N - Show Network Interfaces\r\n");
    iprintf("? - Show This Menu\r\n");
    iprintf("\r\n");
}

void ProcessCommand(char *buffer)
{
    switch (toupper(buffer[0]))
    {
        case 'D':
        case 'd':
            ConfigureESwitch(false);
            break;
        case 'E':
        case 'e':
            ConfigureESwitch(true);
            break;
        case 'I':
        case 'i':
            SetupDialog(0);
            break;
        case 'J':
        case 'j':
            SetupDialog(1);
            break;
        case 'N':
        case 'n':
            ShowNetworkInterfaces();
            break;
        default:
            DisplayMenu();
            break;
    }
}

void UserMain(void * pd)
{
    /* Initialize TCP/IP stack and configure both ethernet Interfaces *
     * and switch vs. independent mode                                */
    InitializeStack();/* Setup the TCP/IP stack buffers etc.. */


    /* Get a DHCP address if needed on every interface */
    GetDHCPAddressIfNecessary();/*Get a DHCP address if needed*/
    /*You may want to add a check for the return value from this function*/
    /*See the function definition in  \nburn\include\dhcpclient.h*/

    OSChangePrio(MAIN_PRIO);/* Change our priority from highest to something in the middle */

    EnableAutoUpdate();/* Enable the ability to update code over the network */

    StartHTTP();/* Start the web serrver */

    ShowNetworkInterfaces();

    iprintf("Application started\n");

    DisplayMenu();
    char buffer[255];
    buffer[0] = '\0';

    while (1)
    {
        if (charavail())
        {
            gets(buffer);
            ProcessCommand(buffer);
            buffer[0] = '\0';
        }
    }
}
